function mediaObject() {
  const postNamePlay = 'play';
  const postNameStop = 'stop';
  const postNameGetUserMedia = 'getUserMedia';
  const typeFunction = 'function';

  function postMessage(name, body) {
    const params = body || {};
    window.webkit.messageHandlers.mediaDevices.postMessage({ name, params });
  }

  const mediaDevicesResponse = {
    getUserMediaSuccess: () => {
      mediaDevicesResponse.getUserMediaResolve(new MediaStream());
    },
    getUserMediaError: (error) => {
      mediaDevicesResponse.getUserMediaReject(error);
    },
  };

  const mediaDevices = {
    getUserMedia: (params) => {
      setUpHTMLMediaElement();
      setUpMediaStream();
      return new Promise((resolve, reject) => {
        mediaDevicesResponse.getUserMediaResolve = resolve;
        mediaDevicesResponse.getUserMediaReject = reject;
        postMessage(postNameGetUserMedia, params);
      });
    },
  };

  function play() {
    postMessage(postNamePlay);
  }

  function stop() {
    postMessage(postNameStop);
  }

  function getTracks() {
    return [];
  }

  function setUpHTMLMediaElement(isFaceScan = true) {
    if (typeof HTMLMediaElement === typeFunction) {
      if (!HTMLMediaElement.prototype.originPlay) {
        HTMLMediaElement.prototype.originPlay = HTMLMediaElement.prototype.play;
      }
      if (isFaceScan) {
        HTMLMediaElement.prototype.play = play;
        return;
      }
      if (HTMLMediaElement.prototype.play === play) {
        HTMLMediaElement.prototype.play = HTMLMediaElement.prototype.originPlay;
      }
    }
  }

  function setUpMediaStream(isFaceScan = true) {
    if (typeof MediaStream === typeFunction) {
      if (!MediaStream.prototype.originStop) {
        MediaStream.prototype.originStop = MediaStream.prototype.stop;
      }
      if (!MediaStream.prototype.originGetTracks) {
        MediaStream.prototype.originGetTracks = MediaStream.prototype.getTracks;
      }
      if (isFaceScan) {
        MediaStream.prototype.stop = stop;
        MediaStream.prototype.getTracks = getTracks;
        return;
      }
      if (MediaStream.prototype.stop === stop) {
        MediaStream.prototype.stop = MediaStream.prototype.originStop;
      }
      if (MediaStream.prototype.getTracks === getTracks) {
        MediaStream.prototype.getTracks = MediaStream.prototype.originGetTracks;
      }
    }
  }

  if (typeof MediaDevices === typeFunction) {
    MediaDevices.prototype.getUserMedia = mediaDevices.getUserMedia;
  }

  function setUpMediaDevices() {
    if (navigator.mediaDevicesResponse) {
      return;
    }
    navigator.mediaDevicesResponse = mediaDevicesResponse;
    if (!navigator.mediaDevices) {
      navigator.mediaDevices = mediaDevices;
    }
    setUpHTMLMediaElement(false);
    setUpMediaStream(false);
  }

  return {
    setUpMediaDevices,
  };
}

mediaObject().setUpMediaDevices();
